/*------------------------------------------------------------------------------
    GALE. Fine-grained parallel Genetic Algorithms for Data Mining.
    Copyright (C) 2002  Xavier Llor <xllora@illigal.ge.uiuc.edu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
------------------------------------------------------------------------------*/


package edu.uiuc.ge.illigal.gale.data;

/**
 * This class is a view over a certain InstanceMemory.
 *
 * @author  Xavier Llor&agrave; &lt;<a href="mailto:xllora@illigal.ge.uiuc.edu">xllora@illigal.ge.uiuc.edu</a>&gt;
 * @version 0.9alpha
 * @since 0.9alpha
 */

public class DataView 
implements OrderedSet
{
	/** The Lower Bound of the view */
	private int iLowerBound;

	/** The Upper Bound of the view */
	private int iUpperBound;

	/** The Ordered data set*/
	private OrderedSet osData;

	/** The number of instances in the view (calculated field) */
	private int iSize;

   //---------------------------------------------------------------------

	/** Builds a view over the OrderSet given.
	 *
    *  @param os The ordered set that contains the data	
	 * @param iLb The lower bound of the view
	 * @param iUb The upper bound of the view
    */
   public DataView ( OrderedSet os, int iLb, int iUb )
   {
		osData      = os;
		iLowerBound = iLb;
		iUpperBound = iUb;
		iSize       = iUb-iLb+1;
   }

   //---------------------------------------------------------------------

	// OrderedSet implementation
   public Instance getInstance ( int iPos )
	{
		if ( iPos>=0 && iPos<iSize )
			return osData.getInstance(iPos+iLowerBound);
		else
			return null;
	}

   //---------------------------------------------------------------------

   // OrderedSet implementation
	public int getSize ()
   {
		return iSize;
	}

   //---------------------------------------------------------------------
}
