/*------------------------------------------------------------------------------
    GALE. Fine-grained parallel Genetic Algorithms for Data Mining.
    Copyright (C) 2002  Xavier Llor <xllora@illigal.ge.uiuc.edu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
------------------------------------------------------------------------------*/


package edu.uiuc.ge.illigal.gale.data;

import java.util.Vector;
import java.util.Random;

/**
 * This class computes the folders for cross-validation.
 * set.
 *
 * @author  Xavier Llor&agrave; &lt;<a href="mailto:xllora@illigal.ge.uiuc.edu">xllora@illigal.ge.uiuc.edu</a>&gt;
 * @version 0.9alpha
 * @since 0.9alpha
 */

public class FoldGenerator
{
   //---------------------------------------------------------------------

   /** Computes the folders for an stratified cross-validation
	 *
	 *  @param os OrderedSet to use
    *  @param iKFolds number of folds
    *  @param iCls number of classes
    *  @param rnd pseudo-random number generator
    *  @return the set of folders
    */
   public static OrderedSet [] stratifiedFolds ( OrderedSet os,
	                                              int iKFolds,
	                                              int iCls,
	                                              Random rnd )
	{
		int i, iTmp;
		final int iMax = os.getSize();
		final int iInsPerFold = iMax/iKFolds;
		Instance ins = null;
		Vector [] vecArr = new Vector[iCls];
		Vector [] vecFld = new Vector[iKFolds];
		float  [] faProp = new float[iCls];
		int    [] iaNum  = new int[iCls];

		// Vectors for Stratification
		for ( i=0 ; i<iCls ; i++ )
			vecArr[i] = new Vector();

		// Folds' Vectors
		for ( i=0 ; i<iKFolds ; i++ )
			vecFld[i] = new Vector();
		
		// Stratifying instances
		for ( i=0 ; i<iMax ; i++ ) {
			ins = os.getInstance(i);
			vecArr[ins.getInstanceClass()].addElement(ins);
		}

		// Computing the class proportions per folder
		for ( i=0 ; i<iCls ; i++ ) {
			faProp[i] = ((float)vecArr[i].size())/iMax;
			iaNum[i]  = (int)(faProp[i]*iInsPerFold);
		}

		// Filling the folders
		for ( i=0 ; i<iKFolds ; i++ )
			for ( int j=0 ; j<iCls ; j++ )
				for ( int k=0 ; k<iaNum[j] ; k++ ) {
					iTmp = rnd.nextInt(vecArr[j].size());
					vecFld[i].addElement(vecArr[j].elementAt(iTmp));
					vecArr[j].remove(iTmp);
				}

		// Checking for missed instances
		for ( i=0 ; i<iCls ; i++ ) {
			iTmp = vecArr[i].size();
			if ( iTmp>0 ) {
				for ( int j=0 ; j<iTmp ; j++ )
					vecFld[rnd.nextInt(iKFolds)].addElement(
						vecArr[i].elementAt(j)
					);
			}
		}

		// Build an array of OrderedSets using the computed folders.

		OrderedSet [] osa = new OrderedSet[iKFolds];
		for ( i=0 ; i<iKFolds ; i++ ) 
			osa[i] = new FoldView(vecFld[i],rnd);

		return osa;
	}

   //---------------------------------------------------------------------
}
