/*------------------------------------------------------------------------------
    GALE. Fine-grained parallel Genetic Algorithms for Data Mining.
    Copyright (C) 2002  Xavier Llor <xllora@illigal.ge.uiuc.edu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
------------------------------------------------------------------------------*/


package edu.uiuc.ge.illigal.gale.data;

import java.util.Vector;

/**
 * This <strong>abstract</strong> class contains the information of a instance. 
 * It is designed to contain real-valued instances.
 *
 * @author  Xavier Llor&agrave; &lt;<a href="mailto:xllora@illigal.ge.uiuc.edu">xllora@illigal.ge.uiuc.edu</a>&gt;
 * @version 0.9alpha
 * @since 0.9alpha
 */

public abstract class Instance
{
   /** Instance length*/
   protected int iLength;

   /** Contains the numerical information of the instance */
   protected float [] faInfo;

   /** Contains the if the data is known or not */
   protected boolean [] baKnown;

   /** Contains the class where the instance belongs */
   protected int iClass;

   //---------------------------------------------------------------------

   /** Returns the class where the instance belongs
	 *
    *  @return The value of the class
    */
   public int getInstanceClass ()
   {
        return iClass;
   }

   //---------------------------------------------------------------------

   /** Returns the numerical value for the element specified
	 *
    *  @param iPos Index of the requested value
    *  @return The value of the requested index
    */
   public float getValue ( int iPos )
   {
        return faInfo[iPos];
   }

   //---------------------------------------------------------------------

   /** Returns if the given position is useful or not
	 *
    *  @param iPos Index of the requested value
    *  @return The value of the requested index
    */
   public boolean isKnown ( int iPos )
   {
        return baKnown[iPos];
   }

   //---------------------------------------------------------------------

   /** Returns the instance length
	 *
    *  @return The instance length
    */
   public int getLength ()
   {
        return iLength;
   }

   //---------------------------------------------------------------------

   /** Converts the instance into a string readable format. It must be
    *  implemented for all subclasses
	 *
    *  @return The Instance information in a String format
    */
   public abstract String toString ();

   //---------------------------------------------------------------------

}
