/*------------------------------------------------------------------------------
    GALE. Fine-grained parallel Genetic Algorithms for Data Mining.
    Copyright (C) 2002  Xavier Llor <xllora@illigal.ge.uiuc.edu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
------------------------------------------------------------------------------*/


package edu.uiuc.ge.illigal.gale.data;

import java.util.StringTokenizer;
import java.util.Vector;


/**
 * This class contains the information of a instance obtained from a String.
 * The information must be separated with coma delimiters.
 *
 * @author  Xavier Llor&agrave; &lt;<a href="mailto:xllora@illigal.ge.uiuc.edu">xllora@illigal.ge.uiuc.edu</a>&gt;
 * @version 0.9alpha
 * @since 0.9alpha
 */

public class StringInstance 
extends Instance
{
   //---------------------------------------------------------------------

	/** The attribute information */
	protected AttributeSet attSet;

   //---------------------------------------------------------------------

   /** Initialize a instance from the String information that it receives.
	 *
    * The instance values are  described by real values, except the last one
    * which is the integer that defines the class where the instance belongs to.
	 *
    *  @param sInstance Contains the information of instance
    *  @param attSet the set of attributes to use
	 *  @throws Exception could not real the instance for the given attribute set
    */
   public StringInstance ( String sInstance, AttributeSet attSet )
	throws Exception
   {
		this.attSet = attSet;
      Vector vecTokens = new Vector();
      StringTokenizer stIn = new StringTokenizer(sInstance,",");

      while ( stIn.hasMoreTokens() ) 
         vecTokens.addElement(stIn.nextToken());

      iLength = attSet.size()-1;
      faInfo  = new float[iLength];
      baKnown = new boolean[iLength];
      for ( int i=0 ; i<iLength ; i++ ) {
			int iType = attSet.attribute(i).type();
         String sT = vecTokens.elementAt(i).toString();
			if ( sT.equals("?") )
				baKnown[i] = false;
			else {
				baKnown[i] = true;
				if ( iType==Attribute.REAL || iType==Attribute.INTEGER ) 
         		faInfo[i] = Float.parseFloat(sT);
				else if ( iType==Attribute.NOMINAL ) 
					faInfo[i] = attSet.attribute(i).tagValue(sT);
				else 
					throw new Exception("Wrong attribute type ("+iType+")"); 
			}
		}
		String st = vecTokens.elementAt(iLength).toString();
		if ( st.equals("?") ) 
			throw new Exception("Class cannot be unknown");
		else {
      	iClass = attSet.attribute(iLength).tagValue( 
		   	             vecTokens.elementAt(iLength).toString()
		      	         );
		}
   }

   //---------------------------------------------------------------------

   /** Converts the instance into a string readable format. The format is
    *  equivalent to the constructor format.
	 *
    *  @return The Instance information in a String format
    */
   public String toString ()
   {
		Attribute a = null;
     	String sRes = "";

     	for ( int i=0 ; i<iLength ; i++ ) {
			a = attSet.attribute(i);
			if ( baKnown[i] ) {
				if ( a.type()==Attribute.REAL )
        			sRes += faInfo[i]+",";
				else if ( a.type()==Attribute.NOMINAL )
        			sRes += a.tag((int)faInfo[i])+",";
				else if ( a.type()==Attribute.INTEGER )
					sRes += (int)faInfo[i]+",";
			}
			else {
				sRes += "?,";
			}
		}
     	sRes += attSet.attribute(iLength).tag(iClass);

     	return sRes;
   }

   //---------------------------------------------------------------------

}
