/*------------------------------------------------------------------------------
    GALE. Fine-grained parallel Genetic Algorithms for Data Mining.
    Copyright (C) 2002  Xavier Llor <xllora@illigal.ge.uiuc.edu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
------------------------------------------------------------------------------*/


package edu.uiuc.ge.illigal.gale.distance;

import edu.uiuc.ge.illigal.gale.data.Instance;
import edu.uiuc.ge.illigal.gale.data.AttributeSet;

/** 
 * Defines a simple interface for metric functions
 *
 * @author  Xavier Llor&agrave; &lt;<a href="mailto:xllora@illigal.ge.uiuc.edu">xllora@illigal.ge.uiuc.edu</a>&gt;
 * @version 0.9alpha 
 * @since   0.9alpha 
 *
 */

public interface Distance
{
	//-------------------------------------------------------------------------

	/** 
	 * Initializes the distance function
	 *
	 * @param att the set of attributes to use
	 * @throws Exception the attributes' types were incompatible
	 */
	public void init ( AttributeSet att )
	throws Exception;
	
	//-------------------------------------------------------------------------

	/**
	 * This function computes the distance between an instance an a float
	 * array starting at the iOff position
	 * 
	 * @param fa the float array to compute
	 * @param iOff The starting array point
	 * @param ins The instance used
	 * @param iLength The number of float to be compared
	 * @return The computed distance
	 */
	public float distance ( float [] fa, 
	                        int iOff,
	                        Instance ins,
	                        int iLength );

	//-------------------------------------------------------------------------

	/**
	 * This function computes the distance between an instance an a float
	 * array starting at the iOff position, only if its required.
	 * 
	 * @param fa the float array to compute
	 * @param ba the boolean array to compute
	 * @param iOff The starting array point
	 * @param ins The instance used
	 * @param iLength The number of float to be compared
	 * @return The computed distance
	 */
	public float distance ( float [] fa, 
	                        boolean [] ba, 
	                        int iOff,
	                        Instance ins,
	                        int iLength );

	//-------------------------------------------------------------------------

	/**
	 * Returns the String representation of the distance function.
	 * 
	 * @return The string class
	 */
	public String toString();

	//-------------------------------------------------------------------------

	/**
	 * Returns the maximal distance for attributes space.
	 * 
	 * @return the distance
	 */
	public float maximalDistance();

	//-------------------------------------------------------------------------

}

