/*------------------------------------------------------------------------------
    GALE. Fine-grained parallel Genetic Algorithms for Data Mining.
    Copyright (C) 2002  Xavier Llor <xllora@illigal.ge.uiuc.edu>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
------------------------------------------------------------------------------*/


package edu.uiuc.ge.illigal.gale.distance;

import edu.uiuc.ge.illigal.gale.data.Instance;
import edu.uiuc.ge.illigal.gale.data.Attribute;
import edu.uiuc.ge.illigal.gale.data.AttributeSet;

/** 
 * Implements a measure for mixed data.
 *
 * @author  Xavier Llor&agrave; &lt;<a href="mailto:xllora@illigal.ge.uiuc.edu">xllora@illigal.ge.uiuc.edu</a>&gt;
 * @version 0.9alpha 
 * @since   0.9alpha 
 *
 */

public class Mixed
implements Distance
{
	//-------------------------------------------------------------------------

	/** Attribute set information */
	protected AttributeSet attCnf;

	/** Maximal distance for attribute set */
	protected float fMaximalDis;

	//-------------------------------------------------------------------------

	// Distance implementation
	public void init ( AttributeSet att )
	throws Exception
	{
		float fAcc = 0;
		float fTmp = 0;

		attCnf = att;

		for ( int i=0, iMax = att.size()-1 ; i<iMax ; i++ ) {
			Attribute a = att.attribute(i);
			if ( a.type()!=Attribute.NOMINAL && 
			     a.type()!=Attribute.REAL &&
			     a.type()!=Attribute.INTEGER ) 
				throw new Exception(att.attribute(i).name()+" not nominal nor real"+
				                    " nor integer valued. Mixed distance could not"+
				                    " be applied");
		}

		fMaximalDis = 1;
	}
	
	//-------------------------------------------------------------------------

	// Distance implementation
	public float distance ( float [] fa, 
	                        int iOff,
	                        Instance ins,
	                        int iLength )
	{
		float    fAcc = 0;
      float    fTmp = 0;
      int      iAtt = 0;
		Attribute att = null;
     
      for ( int i=0, j=iOff ; i<iLength ; i++, j++ ) {
			if ( ins.isKnown(i) ) {
				att = attCnf.attribute(i);
				if ( att.type()==Attribute.NOMINAL ) {
					if ( ((int)fa[j])==((int)ins.getValue(i)) )
						fTmp = 1;
					else 
						fTmp = 0;
					fAcc += fTmp*fTmp;
         		iAtt++;
				}
				else if ( att.type()==Attribute.REAL ||
					       att.type()==Attribute.INTEGER ) {
					fTmp = (fa[j]-ins.getValue(i))/att.distance();
					fAcc += fTmp*fTmp;
         		iAtt++;
				}
			}
      }

      return (float)Math.sqrt(fAcc/iAtt);

	}

	//-------------------------------------------------------------------------

	// Distance implementation
	public float distance ( float [] fa, 
	                        boolean [] ba, 
	                        int iOff,
	                        Instance ins,
	                        int iLength )
	{
		float    fAcc = 0;
      float    fTmp = 0;
      int      iAtt = 0;
		Attribute att = null;
     
      for ( int i=0, j=iOff ; i<iLength ; i++, j++ ) {
			att = attCnf.attribute(i);
      	if ( ba[j] && ins.isKnown(i) ) {
				if ( att.type()==Attribute.NOMINAL ) {
					if ( ((int)fa[j])==((int)ins.getValue(i)) )
						fTmp = 1;
					else 
						fTmp = 0;
					fAcc += fTmp*fTmp;
         		iAtt++;
				}
				else if ( att.type()==Attribute.REAL ||
					       att.type()==Attribute.INTEGER ) {
					fTmp = (fa[j]-ins.getValue(i))/att.distance();
					fAcc += fTmp*fTmp;
         		iAtt++;
				}
         }
		}

      return (float)Math.sqrt(fAcc/iAtt);
	}

	//-------------------------------------------------------------------------

	// Distance implementation
	public String toString()
	{
		return "Mixed";
	}

	//-------------------------------------------------------------------------

	// Distance implementation
	public float maximalDistance ()
	{
		return fMaximalDis;
	}

	//-------------------------------------------------------------------------

}

